@@ -3,6 +3,7 @@
 import random
 from abc import ABC, abstractmethod
 from collections.abc import Iterable, Sized
+from typing import Callable, Optional, Union, List
 
 from datum import error
 from datum.const import MAX_DICE_FACE, MAX_DICE_COUNT, MAX_HIGHEST_COUNT, MAX_LOWEST_COUNT, MAX_DICELET_SIZE
@@ -20,6 +21,7 @@ class ConstComponent(Component):
 
 class Const(Component):
     def __init__(self, value):
+        super().__init__()
         self._value = value
 
     def to_result(self) -> ConstResult:
@@ -35,6 +37,7 @@ class Const(Component):
 
 class Percent(Component):
     def __init__(self, value):
+        super().__init__()
         self._value = value
 
     def to_result(self) -> ConstResult:
@@ -43,6 +46,7 @@ class Percent(Component):
 
 class ConstNeg(ConstComponent):
     def __init__(self, origin):
+        super().__init__()
         self._origin = origin
 
     def to_result(self) -> ConstResult:
@@ -56,6 +60,7 @@ class ConstNeg(ConstComponent):
 
 class ConstBracket(ConstComponent):
     def __init__(self, origin):
+        super().__init__()
         self._origin = origin
 
     def to_result(self) -> ConstResult:
@@ -64,9 +69,15 @@ class ConstBracket(ConstComponent):
     def __str__(self):
         return '({!s})'.format(self._origin)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(ConstBracket, self).set_dice_generator(generator)
+        if isinstance(self._origin, Component):
+            self._origin.set_dice_generator(generator)
+
 
 class ConstCalculation(ConstComponent):
-    def __init__(self, a, operator, b):
+    def __init__(self, a, operator: str, b):
+        super().__init__()
         self._a = a
         self._operator = operator
         self._b = b
@@ -92,9 +103,18 @@ class ConstCalculation(ConstComponent):
     def __str__(self):
         return '{!s} {} {!s}'.format(self._a, self._operator, self._b)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(ConstCalculation, self).set_dice_generator(generator)
+        if isinstance(self._a, Component):
+            self._a.set_dice_generator(generator)
+        if isinstance(self._b, Component):
+            self._b.set_dice_generator(generator)
+
 
 class Dice(ConstComponent):
-    def __init__(self, dice, face, *, highest=None, lowest=None):
+    def __init__(self, dice, face, *,
+                 highest = None, lowest = None):
+        super().__init__()
         self._dice = dice
         self._face = face
         self._highest = highest or None
@@ -106,7 +126,7 @@ class Dice(ConstComponent):
         highest = int(to_value(self._highest)) if self._highest is not None else None
         lowest = int(to_value(self._lowest)) if self._lowest is not None else None
         self._validate(dice, face, highest, lowest)
-        dices = [random.randint(1, face) for _ in range(dice)]
+        dices = [self._generator(face) for _ in range(dice)]
         return DiceResult(dice, face, dices, highest=highest, lowest=lowest)
 
     @staticmethod
@@ -138,6 +158,17 @@ class Dice(ConstComponent):
             return '{}L{!s}'.format(pure, self._lowest)
         return pure
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(Dice, self).set_dice_generator(generator)
+        if isinstance(self._dice, Component):
+            self._dice.set_dice_generator(generator)
+        if isinstance(self._face, Component):
+            self._face.set_dice_generator(generator)
+        if self._highest and isinstance(self._highest, Component):
+            self._highest.set_dice_generator(generator)
+        if self._lowest and isinstance(self._lowest, Component):
+            self._lowest.set_dice_generator(generator)
+
 
 class DiceletComponent(ABC, Sized, Component):
     @abstractmethod
@@ -163,7 +194,8 @@ class DiceletContainer(Iterable, DiceletComponent):
         return list(origin)
 
     def __init__(self, origin):
-        self._container = self._to_list(origin)
+        super().__init__()
+        self._container: List = self._to_list(origin)
         self._len = self._calc_len(self._container)
 
     def __iadd__(self, other):
@@ -193,9 +225,16 @@ class DiceletContainer(Iterable, DiceletComponent):
         pure = ', '.join(str(i) for i in self._container)
         return '{{{}}}'.format(pure)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(DiceletContainer, self).set_dice_generator(generator)
+        for i in self._container:
+            if isinstance(i, Component):
+                i.set_dice_generator(generator)
+
 
 class DiceletBracket(DiceletComponent):
     def __init__(self, origin: DiceletComponent):
+        super().__init__()
         self._origin = origin
 
     def to_result(self):
@@ -207,9 +246,14 @@ class DiceletBracket(DiceletComponent):
     def __str__(self):
         return '({!s})'.format(self._origin)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(DiceletBracket, self).set_dice_generator(generator)
+        self._origin.set_dice_generator(generator)
+
 
 class DiceletNeg(DiceletComponent):
     def __init__(self, origin: DiceletComponent):
+        super().__init__()
         self._origin = origin
 
     def to_result(self):
@@ -223,9 +267,14 @@ class DiceletNeg(DiceletComponent):
     def __str__(self):
         return '-{!s}'.format(self._origin)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(DiceletNeg, self).set_dice_generator(generator)
+        self._origin.set_dice_generator(generator)
+
 
 class DiceletRepeat(DiceletComponent):
     def __init__(self, times, expr):
+        super().__init__()
         self._times = int(to_value(times))
         self._expr = expr
 
@@ -242,6 +291,11 @@ class DiceletRepeat(DiceletComponent):
     def __str__(self):
         return '{}#{!s}'.format(self._times, self._expr)
 
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(DiceletRepeat, self).set_dice_generator(generator)
+        if isinstance(self._expr, Component):
+            self._expr.set_dice_generator(generator)
+
 
 class DiceletCalculation(DiceletComponent):
     @staticmethod
@@ -250,7 +304,8 @@ class DiceletCalculation(DiceletComponent):
             return origin
         return DiceletContainer(origin)
 
-    def __init__(self, a, operator, b):
+    def __init__(self, a, operator: str, b):
+        super().__init__()
         self._a = self._to_container(a)
         self._operator = operator
         self._b = self._to_container(b)
@@ -302,3 +357,10 @@ class DiceletCalculation(DiceletComponent):
 
     def __str__(self):
         return '{!s} {} {!s}'.format(self._a, self._operator, self._b)
+
+    def set_dice_generator(self, generator: Callable[[int], int]) -> None:
+        super(DiceletCalculation, self).set_dice_generator(generator)
+        if isinstance(self._a, Component):
+            self._a.set_dice_generator(generator)
+        if isinstance(self._b, Component):
+            self._b.set_dice_generator(generator)
